/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/** @file artnet/artnet.h */

#ifndef _defined_artnet_h
#define _defined_artnet_h

#include <stdint.h>
// order is important here for osx
#include <sys/types.h>
#include <sys/select.h>

#include <artnet/common.h>

#define ARTNET_ADDRESS_NO_CHANGE	0x7f

#ifdef __cplusplus
extern "C" {
#endif
		
/**
 * An enum for setting the behaviour of a port.
 * Ports can either input data (DMX -> ArtNet) or
 * output (ArtNet -> DMX) data.
 */
typedef enum {
	ARTNET_ENABLE_INPUT = 0x40,	/**< Enables the input for this port */
	ARTNET_ENABLE_OUTPUT = 0x80	/**< Enables the output for this port */
} artnet_port_settings_t ;


typedef enum {
	ARTNET_PC_NONE = 0x00,
	ARTNET_PC_CANCEL = 0x01,
	ARTNET_PC_LED_NORMAL = 0x02,
	ARTNET_PC_LED_MUTE = 0x03,
	ARTNET_PC_LED_LOCATE = 0x04,
	ARTNET_PC_RESET = 0x05,
	ARTNET_PC_MERGE_LTP_O = 0x10,
	ARTNET_PC_MERGE_LTP_1 = 0x11,
	ARTNET_PC_MERGE_LTP_2 = 0x12,
	ARTNET_PC_MERGE_LTP_3 = 0x13,
	ARTNET_PC_MERGE_HTP_0 = 0x50,
	ARTNET_PC_MERGE_HTP_1 = 0x51,
	ARTNET_PC_MERGE_HTP_2 = 0x52,
	ARTNET_PC_MERGE_HTP_3 = 0x53,
	ARTNET_PC_CLR_0 = 0x93,
	ARTNET_PC_CLR_1 = 0x93,
	ARTNET_PC_CLR_2 = 0x93,
	ARTNET_PC_CLR_3 = 0x93,
} artnet_port_command_t ;

/**
 * An enum for the type of data transmitted on a port.
 * As far as I know, only DMX-512 is supported
 */
typedef enum  {
	ARTNET_PORT_DMX = 0x00,		/**< Data is DMX-512 */
	ARTNET_PORT_MIDI = 0x01,	/**< Data is MIDI */
	ARTNET_PORT_AVAB = 0x02,	/**< Data is Avab */
	ARTNET_PORT_CMX = 0x03,		/**< Data is Colortran CMX */
	ARTNET_PORT_ADB = 0x04,		/**< Data is ABD 62.5 */
	ARTNET_PORT_ARTNET = 0x05	/**< Data is ArtNet */
} artnet_port_data_code ;


// defined the status of the firmware transfer
typedef enum  {
	ARTNET_FIRMWARE_BLOCKGOOD = 0x00,
	ARTNET_FIRMWARE_ALLGOOD = 0x01,	
	ARTNET_FIRMWARE_FAIL = 0xff,	
} artnet_firmware_status_code ;



/**
 * An enum for refering to a particular input or output port.
 */
typedef enum {
	ARTNET_INPUT_PORT = 1,		/**< The input port */
	ARTNET_OUTPUT_PORT			/**< The output port */
} artnet_port_dir_t ;

/**
 * Enum describing the type of node
 */
typedef enum {
	ARTNET_SRV,			/**< An ArtNet server (transmitts DMX data) */
	ARTNET_NODE,			/**< An ArtNet node   (dmx reciever) */
	ARTNET_MSRV,			/**< A Media Server */
	ARTNET_RAW			/**< Raw Node - used for diagnostics */
} artnet_node_type ;

/**
 * Enum for the talk-to-me value
 * These values can be &'ed togeather, so for example to set private replies
 * and auto replying use :
 * 	(ARTNET_TTM_PRIVATE & ARTNET_TTM_AUTO)
 */
typedef enum {
	ARTNET_TTM_DEFAULT = 0xFF,	/**< default, ArtPollReplies are broadcast, and nodes won't send a ArtPollReply when conditions change */
	ARTNET_TTM_PRIVATE = 0xFE,	/**< ArtPollReplies aren't broadcast */
	ARTNET_TTM_AUTO = 0xFD		/**< ArtPollReplies are send when node conditions chang */
} artnet_ttm_value_t ;
	
/**
 * Enums for the application defined handlers
 */
typedef enum {
	ARTNET_RECV_HANDLER,		/**< Called on reciept of any ArtNet packet */
	ARTNET_SEND_HANDLER,		/**< Called on transmission of any ArtNet packet */
	ARTNET_POLL_HANDLER,		/**< Called on reciept of an ArtPoll packet */
	ARTNET_REPLY_HANDLER,		/**< Called on reciept of an ArtPollReply packet */
	ARTNET_DMX_HANDLER,		/**< Called on reciept of an ArtDMX packet */
	ARTNET_ADDRESS_HANDLER,		/**< Called on reciept of an ArtAddress packet */
	
	ARTNET_TOD_REQUEST_HANDLER,	/**< Called on reciept of an ArtTodRequest packet */

	ARTNET_TOD_DATA_HANDLER,	/**< Called on reciept of an ArtTodData packet */
	
	ARTNET_IPPROG_HANDLER,		/**< Called on reciept of an ArtIPProg packet */
	ARTNET_FIRMWARE_HANDLER,	/**< Called on reciept of an ArtFirmware packet */
	ARTNET_FIRMWARE_REPLY_HANDLER,	/**< Called on reciept of an ArtFirmwareReply packet */

} artnet_handler_name_t;

/**
 * Describes a remote ArtNet node that has been discovered
 *
 */
typedef struct artnet_node_entry_s {
	uint8_t		ip[4];					/**< The IP address, Network byte ordered*/
	int16_t 	ver;					/**< The firmware version */
	int16_t 	sub;					/**< The subnet address */
	int16_t		oem;					/**< The OEM value */
	uint8_t		ubea;					/**< The UBEA version */
	uint8_t		status;
	uint8_t		etsaman[2];				/**< The ESTA Manufacturer code */
	uint8_t		shortname[ARTNET_SHORT_NAME_LENGTH];	/**< The short node name */
	uint8_t		longname[ARTNET_LONG_NAME_LENGTH];	/**< The long node name */
	uint8_t		nodereport[ARTNET_REPORT_LENGTH];	/**< The node report */
	int16_t		numbports;				/**< The number of ports */
	uint8_t		porttypes[ARTNET_MAX_PORTS];		/**< The type of ports */
	uint8_t		goodinput[ARTNET_MAX_PORTS];
	uint8_t		goodoutput[ARTNET_MAX_PORTS];
	uint8_t		swin[ARTNET_MAX_PORTS];
	uint8_t		swout[ARTNET_MAX_PORTS];
	uint8_t		swvideo;
	uint8_t		swmacro;
	uint8_t		swremote;
	uint8_t		style;
	uint8_t		mac[8];					/**< The MAC address of the node */
} artnet_node_entry_t;

/** A pointer to an artnet_node_entry_t */
typedef artnet_node_entry_t *artnet_node_entry ;

typedef struct {
	char short_name[ARTNET_SHORT_NAME_LENGTH] ;
	char long_name[ARTNET_LONG_NAME_LENGTH] ;
	uint8_t subnet ;
	uint8_t in_ports[ARTNET_MAX_PORTS] ;
	uint8_t out_ports[ARTNET_MAX_PORTS] ;
} artnet_node_config_t ;



/** The local ArtNet node */
typedef void *artnet_node ;

/** A list of remote ArtNet nodes */
typedef void *artnet_node_list ;

// node control functions
extern artnet_node artnet_init(const char *ip, int verbose) ;
extern artnet_node artnet_new(const char *ip, int verbose) ;
extern int artnet_start(artnet_node n) ;
extern int artnet_read(artnet_node n, int timeout) ;
extern int artnet_stop(artnet_node n) ;
int artnet_destroy(artnet_node vn) ;

int artnet_join(artnet_node vn1, artnet_node vn2) ;

// handler functions
extern int artnet_set_handler(artnet_node vn, artnet_handler_name_t handler, int (*fh)(artnet_node n, void *pp, void * d), void* data) ;
extern int artnet_set_dmx_handler(artnet_node vn, int (*fh)(artnet_node n,int port, void *d), void *data );
extern int artnet_set_program_handler(artnet_node vn, int (*fh)(artnet_node n, void *d), void *data );
extern int artnet_set_firmware_handler(artnet_node vn, int (*fh)(artnet_node n, int ubea, uint16_t *data, int length, void *d), void *data );

// send functions
extern int artnet_send_poll(artnet_node n, const char *ip, artnet_ttm_value_t talk_to_me) ;
extern int artnet_send_poll_reply(artnet_node n) ;
extern int artnet_send_dmx(artnet_node n, int port_id, int16_t length, uint8_t *data) ;
extern int artnet_raw_send_dmx(artnet_node vn, uint8_t addr, int16_t length, uint8_t *data) ;
extern int artnet_send_address(artnet_node n, artnet_node_entry e,const char *shortName, const char *longName, uint8_t inAddr[ARTNET_MAX_PORTS], uint8_t outAddr[ARTNET_MAX_PORTS], uint8_t subAddr, artnet_port_command_t cmd ) ;

extern int artnet_send_input(artnet_node n, artnet_node_entry e, uint8_t settings[ARTNET_MAX_PORTS]) ;

extern int artnet_send_firmware(artnet_node vn, artnet_node_entry e, int ubea, uint16_t *data, int length, int (*fh)(artnet_node n, artnet_firmware_status_code code, void *d), void *user_data ) ;
extern int artnet_send_firmware_reply(artnet_node vn, artnet_node_entry e, artnet_firmware_status_code code) ;


// recv functions
extern uint8_t *artnet_read_dmx(artnet_node n, int port_id, int *length) ;

// state changing functions
extern int artnet_set_node_type(artnet_node n, artnet_node_type type) ;
extern int artnet_set_short_name(artnet_node vn, const char *name) ;
extern int artnet_set_long_name(artnet_node n, const char *name) ;

//port manipulation functions
extern int artnet_set_port_type(artnet_node n, int id, artnet_port_settings_t settings, artnet_port_data_code data ) ;
extern int artnet_set_port_addr(artnet_node n, int id, artnet_port_dir_t dir, uint8_t addr) ;
extern int artnet_get_universe_addr(artnet_node n, int id, artnet_port_dir_t dir) ;
extern int artnet_set_subnet_addr(artnet_node n, uint8_t subnet) ;

//node list functions
extern artnet_node_list artnet_get_nl(artnet_node n) ;
extern artnet_node_entry artnet_nl_first(artnet_node_list nl) ;
extern artnet_node_entry artnet_nl_next(artnet_node_list nl) ;
extern int artnet_nl_get_length(artnet_node_list nl) ;

// misc
extern int artnet_dump_config(artnet_node n) ;
extern int artnet_get_config(artnet_node n, artnet_node_config_t *config) ;
extern int artnet_get_sd(artnet_node n, int socket) ;
extern int artnet_set_fdset(artnet_node vn, fd_set *fdset) ;
#endif

#ifdef __cplusplus
}
#endif

